package src.server;

import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.LoginException;
import java.util.*;

/**
 * Extends the built-in class of JAAS that makes the yes/no decission 
 * related to the user authentication.
 * LoginModule authenticates users from credentials, and adds the 
 * permissions that were listed for that user in the policy.
 */

public class LoginModule implements javax.security.auth.spi.LoginModule {

	private boolean debug = false;
	
	private Subject subject;
	private MyPrincipal entity;
	private CallbackHandler callbackHandler;
	private Map sharedState;
	private Map options;
	
	/* Table holds credentials of the registered users.
	 * It holds their usernames and their passwords. 
	 */
	private Hashtable userList;
	
	// tracking the authentication status
	private static final int NOT = 0, OK = 1, COMMIT = 2;
	private int status;
	
	// current user
	private String username;
	private char[] password;
	
	//CONSTRUCTOR
	protected LoginModule() {
		userList = new Hashtable();
		
		// Username & passwords of registered users in the clinic.
		// Inserted for testing of the application.
		userList.put("admin1", "admin1"); 
		userList.put("admin2", "admin2");
		// users with Administration priveleges
		userList.put("med1", "med2"); 
		userList.put("med2", "med2");
		// users with Medical staff priveleges
	}

	/*
	 * This method is called if the LoginContext's overall authentication 
	 * failed. Cleans up any state if any of modules succeded but the overal
	 * authentication doesnt succed.
	 */
	public boolean abort() throws LoginException {
		if(status == NOT) {
		    return false;
		} else if( status == OK ) {
		    // login succeeded but overall authentication failed
		    username = null;
		    if( password != null ) password = null;
		    entity = null;
		} else {
		    // overall authentication succeeded and commit succeeded,
		    // but someone else's commit failed
		    logout();
		}//end if/else
		status = NOT;
		return true;
	}

	/*
	 * Called if the overall authentication succeeds. Associates a
     * MyPrincipal object with the Subject located in the LoginModule.
	 */
	public boolean commit() throws LoginException {
		if(status == NOT || subject == null) {
		    return false;
		} else {
		    // add a Principal (authenticated identity) to the Subject
	
		    // assume the user we authenticated is the MyPrincipal
		    entity = new MyPrincipal(username);
		    Set entities = subject.getPrincipals();
		    if( !entities.contains(entity) )
				entities.add(entity);
	
		    // in any case, clean out state
		    username = null;
		    password = null;
	
		    status = COMMIT;
		    return true;
		}
	}

	/**
     * Initialize the LoginModule
     */
	public void initialize(Subject subject, CallbackHandler cbH, 
									Map sharedState, Map options) {
		
		status = NOT;
		this.subject = subject;
		this.callbackHandler = cbH;
		this.sharedState = sharedState;
		this.options = options;
	
	}

	/**
     * Authenticate the user based on provided credentials. 
     * If the credential matches the expected one,
     * authentication succeeds - otherwise it doesnt.
     */
	public boolean login() throws LoginException {
		if( callbackHandler == null )
		    throw new LoginException( "Error: no CallbackHandler " +
		    				"available to retrieve user credentials");
	
		Callback[] callbacks = new Callback[2];
		callbacks[0] = new NameCallback( "\nuser name: " );
		callbacks[1] = new PasswordCallback( "password: ", false );
		
		try {
			callbackHandler.handle(callbacks); 
			//get the user credentials
		    username = ((NameCallback)callbacks[0]).getName();
		    
		    char[] tmpPassword = 
		    	((PasswordCallback)callbacks[1]).getPassword();
		    
		    if (tmpPassword == null) 
		    	// treat a NULL password as an empty password
		    	
			tmpPassword = new char[0];
		    password = new char[tmpPassword.length];
		    
		    System.arraycopy( tmpPassword, 0,password, 0, 
		    							tmpPassword.length );
		    ((PasswordCallback)callbacks[1]).clearPassword(); 
		    //wipe out occurrences in memory
		} 
		catch( java.io.IOException ioe ) {
		    throw new LoginException(ioe.toString());
		} 
		catch( UnsupportedCallbackException uce ) {
		    throw new LoginException( "Error: " + 
		    			uce.getCallback().toString() +
						" not available to authenticate user." );
		}
		
		
		// Lookup the given credentials in our HashMap to see if they're 
		// legitimate users of the clinic		
		if (lookupUser(username, password))
			// because the given username and passwords is
			// found in list, authentication succeeded!
			return true;
			else
				// authentication failed
				return false;
	}
	
	/**
	 * Check if username and password exist in the userList HashTable.
	 * 
	 * @param 	usr		username provided by the client
	 * @param 	pwd		password provided by the client
	 * @return 	true 	if username and password was correct
	 * 			false	if username and/or password incorrect
	 */
	private boolean lookupUser(String usr, char[] pwd) {
		
		if(userList.containsKey(usr)) {
			String key = (String) userList.get(usr);
			// Retrive that username
			if((userList.get(key)).toString().toCharArray()==pwd) {
				// The password maches. 
				// This is a valid user.
				key = null;//Clean up the String location in the memory
						   //containing the password of the user;
				return true;
			}			
		}
		return false;
		// Username does not exist
	}


    /**
     * Logout the user.
     * Removes the MyPrincipal added by the .commit() method.
	 */

	public boolean logout() throws LoginException {
		
		subject.getPrincipals().remove(entity);
		status = NOT;
		username = null;
		
		if( password != null ) password = null;
		entity = null;
		return true;
	}

}
