package src.server;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import javax.security.auth.Subject;

import java.util.ArrayList;
import java.util.Date;

import src.permissionvalidation.*;
import src.common.ServerInterface;

/**
 * Proxy implements the same interface as the server.
 * 
 * Protects the server from direct calls by clients. All calls by 
 * clients have to pass security checks in the proxy.
 * Hides the implementation of the actual methods on the server.
 * 
 * All the methods check is the caller has permissions, and only then
 * call methods for him. If the user doesnt have permission they throw
 * a SecurityException.
 */

public class ServerProxy extends UnicastRemoteObject 
										implements ServerInterface {

	// A reference to the REAL server
	private /* spec_public */ ServerInterface theServer;
	// The particular user that is associated to this proxy
	private /* spec_public */ Subject theUser;

	//@ public ghost String state;
	
	// Constructor
	/*@ requires 		true;
	  @ ensures			subject!=null && theServer!=null;
	  @ signals_only	RemoteException;
	  @*/
	protected ServerProxy(Subject subject, ServerInterface theServer) 
											throws RemoteException {
		this.theServer = theServer;
		this.theUser = subject;
	}	
	/*@ invariant (subject!=null && theServer!=null) &&
	  @			  (state==VERIFY_AUTHORIZATION || state==CALL_METHOD); 
	  @*/
	
	/*@ constraint
	  @		(state==VERIFY_AUTHORIZATION ==> 
	  @				assert \old(state)==REQUEST_METHOD) &&
	  @		(state==CALL_METHOD ==> \old(state)==VERIFY_AUTHORIZATION);
	  @*/
	
	/*@	normal_behavior
	  @		requires	cpr!=null && fName!=null && lName!=null && 
	  @										adress!=null && DOB!=null; 
	  @ 	ensures		\only_called(checkPermission,insert);  
	  @						
	  @ also
	  @	  exceptional_behavior
	  @		 signals_only 	RemoteException, SecurityException;
	  @*/
	public boolean insert(String cpr, String fName, 
				String lName, String adress, Date DOB) 
							throws RemoteException, SecurityException {
		//@ set state==VERIFY_AUTHORIZATION; 
		checkPermission("insert");
		//@ set state==CALL_METHOD;
		return theServer.insert(cpr, fName, lName, adress, DOB);
	}
	
	/*@	normal_behavior
	  @		requires	cpr!=null; 
	  @ 	ensures		\only_called(checkPermission, findP) && 
	  @							\result=!null && \fresh(\result);
	  @ also
	  @	  exceptional_behavior
	  @		 signals_only 	RemoteException, SecurityException;
	  @*/
	public String findP(String cpr) 
					throws RemoteException, SecurityException{
		//@ set state==VERIFY_AUTHORIZATION;
		checkPermission("findP");
		//@ set state==CALL_METHOD;
		return theServer.findP(cpr);
	}
	
	/*@	normal_behavior
	  @		requires	cpr!=null; 
	  @ 	ensures		\only_called(checkPermission, editP) &&
	  @									 \fresh(\result);
	  @ also
	  @	  exceptional_behavior
	  @		 signals_only 	RemoteException, SecurityException;
	  @*/
	public boolean editP(String cpr, String fName, String lName, 
							String adress) throws RemoteException, 
												SecurityException  {
		//@ set state==VERIFY_AUTHORIZATION;
		checkPermission("editP");
		//@ set state==CALL_METHOD;
		return theServer.editP(cpr, fName, lName, adress);
	}
	
	/*@	normal_behavior
	  @		requires	cpr!=null && date!=null && test!=null && 
	  @ 							comment!=null && totalCost!=null;
	  @		ensures		\only_called(checkPermission, 
	  @							addMedicalRecords)&& \fresh(\result);
	  @ also
	  @   exceptional_behavior
	  @		 signals_only 	RemoteException, SecurityException;
	  @*/
	public boolean addMedicalRecords(String cpr, Date date, 
				ArrayList test, String comment, Double totalCost) 
						throws RemoteException, SecurityException {
		
		//@ set state==VERIFY_AUTHORIZATION;
		checkPermission("addMedicalRecords");
		//@ set state==CALL_METHOD;
		return theServer.addMedicalRecords(cpr, date, 
									test, comment, totalCost);
	}
	
	/*@	normal_behavior
	  @		requires	cpr!=null;
	  @		ensures		\only_called(checkPermission, listRecords) &&
	  @								\fresh(\result);
	  @ also
	  @   exceptional_behavior
	  @		 signals_only 	RemoteException, SecurityException;
	  @*/
	public ArrayList listRecords(String cpr) 
			throws RemoteException, SecurityException {
		
		//@ set state=VERIFY_AUTHORIZATION;
		checkPermission("listRecords");
		//@ set state=CALL_METHOD;
		return theServer.listRecords(cpr);
	}
	

	/**
	   * Check if the current client can call a certain method.
	   * The check is made through JAAS and its policy file.
	   * 
	   * @param	methodName	The name of the method to verify if 
	   * 					the user has permission to run.
	   */
	/*@ normal_behavior
	  @		requires	methodName!=null && theUser!=null;
	  @ 	assignable	\nothing;
	  @		ensures		\only_called(doAs);
	  @ also
	  @	  exceptional_behavior	
	  @		 signals_only 	SecurityException;
	  @*/
	private void checkPermission(String methodName) 
											throws SecurityException{
		Subject.doAs(theUser, new MethodcallValidate(methodName));
	}
	
}
